==> nut-17.5.tar.gz: nut-17.5/fltk/PCF.cc <==
/* Class PCF */

/*
    NUT nutrition software 
    Copyright (C) 1996-2012 by Jim Jozwiak.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "PCF.h"
#include "Nut.h"
#include "db.h"
#include <FL/Fl_Tooltip.H>
#include <stdio.h>

static float PCF_matrix[100][4];
static int PCF_lock[100];
static MealFood *PCF_lock_holder[100];
static int PCF_count;
static float ptot, ctot, ftot;
static bool need_to_save_meal_db;

void clear_out_pcfs(void)
{                               
PCF_count = 0;
}

void save_meal_db(void)
{
if (!need_to_save_meal_db) return;
if (Fl::event_buttons() == 0)
 {
 write_meal_db();
 need_to_save_meal_db = 0;
 redraw_whole_schmeer();
 }
}

void no_op(int i) {}

void p_func(int i)
{
float change;
change = ptot / PCF_matrix[i][1];
ptot = 0;
PCF_matrix[i][0] -= change;
ctot -= change * PCF_matrix[i][2];
ftot -= change * PCF_matrix[i][3];
}

void c_func(int i)
{
float change;
change = ctot / PCF_matrix[i][2];
ctot = 0;
PCF_matrix[i][0] -= change;
ptot -= change * PCF_matrix[i][1];
ftot -= change * PCF_matrix[i][3];
}

void f_func(int i)
{
float change;
change = ftot / PCF_matrix[i][3];
ftot = 0;
PCF_matrix[i][0] -= change;
ptot -= change * PCF_matrix[i][1];
ctot -= change * PCF_matrix[i][2];
}

void pcf_cb(Fl_Widget *which, void *who)
{
PCF *who_we_are = (PCF *) who;
Fl_Light_Button *button = (Fl_Light_Button *) which;
if (button->value() == 1) who_we_are->only_one(button);
else who_we_are->update_PCF_matrix();
}

PCF::PCF (int x, int y, int w, int h, Fl_Color widgetcolor, MealFood *mealfood) : Fl_Pack (x, y, w, h)
{
mf = mealfood;
this->type(Fl_Pack::HORIZONTAL);
this->tooltip("Automatic Portion Control:  Specify a food as the Protein, Carb, or Fat food and its quantity is adjusted to meet that DV for the entire meal");
p = new Fl_Light_Button(x, y, w/3, h, "P");
p->labelfont(FL_BOLD);
p->color(fl_lighter(widgetcolor));
p->callback(pcf_cb,this);
c = new Fl_Light_Button(x, y, w/3, h, "C");
c->labelfont(FL_BOLD);
c->color(fl_lighter(widgetcolor));
c->callback(pcf_cb,this);
f = new Fl_Light_Button(x, y, w/3, h, "F");
f->labelfont(FL_BOLD);
f->color(fl_lighter(widgetcolor));
f->callback(pcf_cb,this);
this->end();
}

void PCF::only_one(Fl_Light_Button *button)
{
MealFood *mealfood;
if (button != p) p->value(0); 
if (button != c) c->value(0); 
if (button != f) f->value(0); 
PCF_count = parent()->parent()->children();
for (int i = 0; i < PCF_count; i++)
 {
 mealfood = (MealFood *) (parent()->parent()->child(i));
 if (mealfood != mf)
  {
  if (button == p) mealfood->pcf->p->value(0);
  else if (button == c) mealfood->pcf->c->value(0);
  else mealfood->pcf->f->value(0);
  if (mf->mffoodptr == mealfood->mffoodptr)
   {
   mealfood->pcf->p->value(0);
   mealfood->pcf->c->value(0);
   mealfood->pcf->f->value(0);
   }
  }
 mealfood->fill_PCF_matrix(i, &PCF_matrix[i][0], &PCF_lock[i], &PCF_lock_holder[i]);
 }
PCF::PCF_computation();
}

void PCF::update_PCF_matrix(void)
{
MealFood *mealfood;
PCF_count = parent()->parent()->children();
for (int i = 0; i < PCF_count; i++)
 {
 mealfood = (MealFood *) (parent()->parent()->child(i));
 mealfood->fill_PCF_matrix(i, &PCF_matrix[i][0], &PCF_lock[i], &PCF_lock_holder[i]);
 }
PCF_computation();
}

void PCF::update_PCF_matrix_without_computation(void)
{
MealFood *mealfood;
PCF_count = parent()->parent()->children();
for (int i = 0; i < PCF_count; i++)
 {
 mealfood = (MealFood *) (parent()->parent()->child(i));
 mealfood->fill_PCF_matrix(i, &PCF_matrix[i][0], &PCF_lock[i], &PCF_lock_holder[i]);
 }
}

void PCF::update_PCF_matrix_except_yourself(void)
{
MealFood *mealfood;
int count = -1;
PCF_count = parent()->parent()->children();
for (int i = 0; i < PCF_count; i++)
 {
 mealfood = (MealFood *) (parent()->parent()->child(i));
 if (mf == mealfood) continue;
 count++;
 mealfood->fill_PCF_matrix(count, &PCF_matrix[count][0], &PCF_lock[count], &PCF_lock_holder[count]);
 }
PCF_count--;
PCF_computation();
}

void PCF::gram_change(int index, double newval)
{
PCF_matrix[index][0] = newval;
PCF_computation();
}

void PCF::refresh_values(void)
{
for (int i = 0; i < PCF_count; i++) PCF_lock_holder[i]->non_recursive_gram_change(PCF_matrix[i][0]);
}

void PCF::dv_change(void)
{
PCF_computation();
}

void PCF::PCF_computation()
{
if (PCF_count == 0)
 {
 need_to_save_meal_db = 1;
 Fl::add_timeout(0.5, (Fl_Timeout_Handler)save_meal_db);
 return;
 }
typedef void (*xfunc)(int);
xfunc pfunc = &no_op, cfunc = &no_op, ffunc = &no_op;
int i, p = -1, c = -1, f = -1, iterations[] = {0,1,50,100}, theseits = 0;
ptot = -DV[PROCNT] / options.mealsperday; 
ctot = -DV[CHO_NONFIB] / options.mealsperday; 
ftot = -DV[FAT] / options.mealsperday;
for (i = 0; i < PCF_count; i++)
 {
 if (PCF_lock[i] == 0)
  {
  ptot += PCF_matrix[i][0] * PCF_matrix[i][1];
  ctot += PCF_matrix[i][0] * PCF_matrix[i][2];
  ftot += PCF_matrix[i][0] * PCF_matrix[i][3];
  }
 else switch (PCF_lock[i])
  {
  case 1 : p = i;
           pfunc = &p_func;
           theseits++;
           PCF_matrix[i][0] = 0;
           ptot += PCF_matrix[i][0] * PCF_matrix[i][1];
           ctot += PCF_matrix[i][0] * PCF_matrix[i][2];
           ftot += PCF_matrix[i][0] * PCF_matrix[i][3];
           break;
  case 2 : c = i;
           cfunc = &c_func;
           theseits++;
           PCF_matrix[i][0] = 0;
           ptot += PCF_matrix[i][0] * PCF_matrix[i][1];
           ctot += PCF_matrix[i][0] * PCF_matrix[i][2];
           ftot += PCF_matrix[i][0] * PCF_matrix[i][3];
           break;
  case 3 : f = i;
           ffunc = &f_func; 
           theseits++;
           PCF_matrix[i][0] = 0;
           ptot += PCF_matrix[i][0] * PCF_matrix[i][1];
           ctot += PCF_matrix[i][0] * PCF_matrix[i][2];
           ftot += PCF_matrix[i][0] * PCF_matrix[i][3];
           break;
  }
// Fl::check();
 }
for (i = 0; i < iterations[theseits]; i++)
 {
 pfunc(p);
 cfunc(c);
 ffunc(f);
// Fl::check();
 }
for (i = 0; i < PCF_count; i++)
 {
 if (! (PCF_matrix[i][0] == PCF_matrix[i][0])) PCF_matrix[i][0] = 0;
 PCF_lock_holder[i]->non_recursive_gram_change(PCF_matrix[i][0]);
 }
need_to_save_meal_db = 1;
Fl::add_timeout(0.5, (Fl_Timeout_Handler)save_meal_db);
//Fl::check();
update_analysis();
}

==> nut-17.8.tar.gz: nut-17.8/fltk/PCF.cc <==
/* Class PCF */

/*
    NUT nutrition software 
    Copyright (C) 1996-2012 by Jim Jozwiak.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "PCF.h"
#include "Nut.h"
#include "db.h"
#include <FL/Fl_Tooltip.H>
#include <stdio.h>

static float PCF_matrix[100][4];
static int PCF_lock[100];
static MealFood *PCF_lock_holder[100];
static int PCF_count;
static float ptot, ctot, ftot;
static bool need_to_save_meal_db;

void clear_out_pcfs(void)
{                               
PCF_count = 0;
}

void save_meal_db(void)
{
if (!need_to_save_meal_db) return;
if (Fl::event_buttons() == 0)
 {
 write_meal_db();
 need_to_save_meal_db = 0;
 redraw_whole_schmeer();
 }
}

void no_op(int i) {}

void p_func(int i)
{
float change;
change = ptot / PCF_matrix[i][1];
ptot = 0;
PCF_matrix[i][0] -= change;
ctot -= change * PCF_matrix[i][2];
ftot -= change * PCF_matrix[i][3];
}

void c_func(int i)
{
float change;
change = ctot / PCF_matrix[i][2];
ctot = 0;
PCF_matrix[i][0] -= change;
ptot -= change * PCF_matrix[i][1];
ftot -= change * PCF_matrix[i][3];
}

void f_func(int i)
{
float change;
change = ftot / PCF_matrix[i][3];
ftot = 0;
PCF_matrix[i][0] -= change;
ptot -= change * PCF_matrix[i][1];
ctot -= change * PCF_matrix[i][2];
}

void pcf_cb(Fl_Widget *which, void *who)
{
PCF *who_we_are = (PCF *) who;
Nut_LightButton *button = (Nut_LightButton *) which;
if (button->value() == 1) who_we_are->only_one(button);
else who_we_are->update_PCF_matrix();
}

PCF::PCF (int x, int y, int w, int h, Fl_Color widgetcolor, MealFood *mealfood) : Fl_Group (x, y, w, h)
{
mf = mealfood;
this->tooltip("Automatic Portion Control:  Specify a food as the Protein, Carb, or Fat food and its quantity is adjusted to meet that DV for the entire meal");
p = new Nut_LightButton(x, y, w/3, h, "P");
p->labelfont(FL_BOLD);
p->color(fl_lighter(widgetcolor));
p->callback(pcf_cb,this);
c = new Nut_LightButton(x+w/3, y, w/3, h, "C");
c->labelfont(FL_BOLD);
c->color(fl_lighter(widgetcolor));
c->callback(pcf_cb,this);
f = new Nut_LightButton(x+2*w/3, y, w/3, h, "F");
f->labelfont(FL_BOLD);
f->color(fl_lighter(widgetcolor));
f->callback(pcf_cb,this);
this->end();
}

void PCF::only_one(Nut_LightButton *button)
{
MealFood *mealfood;
if (button != p) p->value(0); 
if (button != c) c->value(0); 
if (button != f) f->value(0); 
PCF_count = parent()->parent()->children();
for (int i = 0; i < PCF_count; i++)
 {
 mealfood = (MealFood *) (parent()->parent()->child(i));
 if (mealfood != mf)
  {
  if (button == p) mealfood->pcf->p->value(0);
  else if (button == c) mealfood->pcf->c->value(0);
  else mealfood->pcf->f->value(0);
  if (mf->mffoodptr == mealfood->mffoodptr)
   {
   mealfood->pcf->p->value(0);
   mealfood->pcf->c->value(0);
   mealfood->pcf->f->value(0);
   }
  }
 mealfood->fill_PCF_matrix(i, &PCF_matrix[i][0], &PCF_lock[i], &PCF_lock_holder[i]);
 }
PCF::PCF_computation();
}

void PCF::update_PCF_matrix(void)
{
MealFood *mealfood;
PCF_count = parent()->parent()->children();
for (int i = 0; i < PCF_count; i++)
 {
 mealfood = (MealFood *) (parent()->parent()->child(i));
 mealfood->fill_PCF_matrix(i, &PCF_matrix[i][0], &PCF_lock[i], &PCF_lock_holder[i]);
 }
PCF_computation();
}

void PCF::update_PCF_matrix_without_computation(void)
{
MealFood *mealfood;
PCF_count = parent()->parent()->children();
for (int i = 0; i < PCF_count; i++)
 {
 mealfood = (MealFood *) (parent()->parent()->child(i));
 mealfood->fill_PCF_matrix(i, &PCF_matrix[i][0], &PCF_lock[i], &PCF_lock_holder[i]);
 }
}

void PCF::update_PCF_matrix_except_yourself(void)
{
MealFood *mealfood;
int count = -1;
PCF_count = parent()->parent()->children();
for (int i = 0; i < PCF_count; i++)
 {
 mealfood = (MealFood *) (parent()->parent()->child(i));
 if (mf == mealfood) continue;
 count++;
 mealfood->fill_PCF_matrix(count, &PCF_matrix[count][0], &PCF_lock[count], &PCF_lock_holder[count]);
 }
PCF_count--;
PCF_computation();
}

void PCF::gram_change(int index, double newval)
{
PCF_matrix[index][0] = newval;
PCF_computation();
}

void PCF::refresh_values(void)
{
for (int i = 0; i < PCF_count; i++) PCF_lock_holder[i]->non_recursive_gram_change(PCF_matrix[i][0]);
}

void PCF::dv_change(void)
{
PCF_computation();
}

void PCF::PCF_computation()
{
if (PCF_count == 0)
 {
 need_to_save_meal_db = 1;
 Fl::add_timeout(0.5, (Fl_Timeout_Handler)save_meal_db);
 return;
 }
typedef void (*xfunc)(int);
xfunc pfunc = &no_op, cfunc = &no_op, ffunc = &no_op;
int i, p = -1, c = -1, f = -1, iterations[] = {0,1,50,100}, theseits = 0;
ptot = -DV[PROCNT] / options.mealsperday; 
ctot = -DV[CHO_NONFIB] / options.mealsperday; 
ftot = -DV[FAT] / options.mealsperday;
for (i = 0; i < PCF_count; i++)
 {
 if (PCF_lock[i] == 0)
  {
  ptot += PCF_matrix[i][0] * PCF_matrix[i][1];
  ctot += PCF_matrix[i][0] * PCF_matrix[i][2];
  ftot += PCF_matrix[i][0] * PCF_matrix[i][3];
  }
 else switch (PCF_lock[i])
  {
  case 1 : p = i;
           pfunc = &p_func;
           theseits++;
           PCF_matrix[i][0] = 0;
           ptot += PCF_matrix[i][0] * PCF_matrix[i][1];
           ctot += PCF_matrix[i][0] * PCF_matrix[i][2];
           ftot += PCF_matrix[i][0] * PCF_matrix[i][3];
           break;
  case 2 : c = i;
           cfunc = &c_func;
           theseits++;
           PCF_matrix[i][0] = 0;
           ptot += PCF_matrix[i][0] * PCF_matrix[i][1];
           ctot += PCF_matrix[i][0] * PCF_matrix[i][2];
           ftot += PCF_matrix[i][0] * PCF_matrix[i][3];
           break;
  case 3 : f = i;
           ffunc = &f_func; 
           theseits++;
           PCF_matrix[i][0] = 0;
           ptot += PCF_matrix[i][0] * PCF_matrix[i][1];
           ctot += PCF_matrix[i][0] * PCF_matrix[i][2];
           ftot += PCF_matrix[i][0] * PCF_matrix[i][3];
           break;
  }
// Fl::check();
 }
for (i = 0; i < iterations[theseits]; i++)
 {
 pfunc(p);
 cfunc(c);
 ffunc(f);
// Fl::check();
 }
for (i = 0; i < PCF_count; i++)
 {
 if (! (PCF_matrix[i][0] == PCF_matrix[i][0])) PCF_matrix[i][0] = 0;
 PCF_lock_holder[i]->non_recursive_gram_change(PCF_matrix[i][0]);
 }
need_to_save_meal_db = 1;
Fl::add_timeout(0.5, (Fl_Timeout_Handler)save_meal_db);
//Fl::check();
update_analysis();
}
